SystemOrganization addCategory: #'Refactoring-Environment'!

Object subclass: #RBBrowserEnvironment
	instanceVariableNames: 'label searchStrings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:26'!
& anEnvironment 
	"If we or anEnvironment includes everything, then just include the other environment (optimization)"

	self isSystem ifTrue: [^anEnvironment].
	anEnvironment isSystem ifTrue: [^self].
	^RBAndEnvironment onEnvironment: self and: anEnvironment! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'lr 2/8/2009 12:28'!
addSearchString: aString 
	searchStrings isNil ifTrue: [
		searchStrings := SortedCollection sortBlock: [ :a :b | 
			(a indexOf: $: ifAbsent: [ a size ]) > (b indexOf: $: ifAbsent: [ b size ]) ] ].
	(searchStrings includes: aString)
		ifFalse: [ searchStrings add: aString ]! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 6/10/2010 15:29'!
allClasses
	"Answer all the unique non-metaclasses of all the classes and metaclasses in this environment."

	| classes |
	classes := IdentitySet new: 4096.
	self classesDo: [ :each | 
		classes add: each theNonMetaClass ].
	^ classes asArray! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'CamilloBruni 9/14/2012 02:16'!
allClassesAndTraits
	" compatibility method with SystemDictionary "
	^ self allClasses! !

!RBBrowserEnvironment methodsFor: 'private' stamp: 'lr 3/20/2011 11:18'!
allClassesDo: aBlock
	self systemDictionary allClassesDo: [ :each | aBlock value: each; value: each class ]! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 6/10/2010 15:29'!
allMetaClasses
	"Answer all the unique non-metaclasses of all metaclasses in this environment."

	| classes |
	classes := IdentitySet new: 4096.
	self classesDo: [ :each | 
		each isMeta
			ifTrue: [ classes add: each theNonMetaClass ] ].
	^ classes asArray! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 6/10/2010 15:29'!
allNonMetaClasses
	"Answer all the unique non-metaclasses of all of all the non-metaclasses in this environment."

	| classes |
	classes := IdentitySet new: 4096.
	self classesDo: [ :each | 
		each isMeta
			ifFalse: [ classes add: each ] ].
	^ classes asArray! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'lr 9/8/2011 20:32'!
asSelectorEnvironment
	^(RBClassEnvironment onEnvironment: self classes: self classes) asSelectorEnvironment! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 1/23/2010 17:10'!
associationAt: aKey
	^ self associationAt: aKey ifAbsent: [ self error: aKey printString , ' not found' ]! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 3/20/2011 11:18'!
associationAt: aKey ifAbsent: aBlock
	| association class |
	association := self systemDictionary
		associationAt: aKey
		ifAbsent: [ ^ aBlock value ].
	class := association value isBehavior
		ifTrue: [ association value ]
		ifFalse: [ association value class ].
	^ ((self includesClass: class) or: [ self includesClass: class class ])
		ifTrue: [ association ]
		ifFalse: [ nil ]! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 2/26/2009 14:46'!
at: aKey
	^ self at: aKey ifAbsent: [ self error: aKey printString , ' not found' ]! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 1/23/2010 17:15'!
at: aKey ifAbsent: aBlock
	| association |
	association := self
		associationAt: aKey 
		ifAbsent: [ nil ].
	^ association isNil
		ifTrue: [ aBlock value ]
		ifFalse: [ association value ]! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'lr 8/9/2011 21:00'!
categories
	^ self systemDictionary organization categories select: [ :each | self includesCategory: each ]! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 11/25/2009 08:31'!
classNames
	| names |
	names := IdentitySet new: 4096.
	self classesDo: [ :each | names add: each theNonMetaClass name ].
	^ names! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'lr 8/9/2011 21:00'!
classNamesFor: aCategoryName
	^ (self systemDictionary organization listAtCategoryNamed: aCategoryName) select: [ :each |
		| class |
		class := self systemDictionary at: each ifAbsent: [ nil ].
		class notNil 
			and: [ (self includesClass: class)
			or: [ self includesClass: class class ] ] ]! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
classVarRefsTo: instVarName in: aClass 
	^ RBVariableEnvironment 
		on: self
		referencesToClassVariable: instVarName
		in: aClass! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
classVariablesFor: aClass 
	^aClass classVarNames! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 2/26/2009 14:45'!
classes
	| classes |
	classes := IdentitySet new: 4096.
	self classesDo: [ :each | classes add: each ].
	^ classes! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
classesAndSelectorsDo: aBlock 
	self classesDo: 
			[:class | 
			self selectorsForClass: class do: [:sel | aBlock value: class value: sel]]! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 2/26/2009 14:45'!
classesDo: aBlock 
	self allClassesDo: [ :each |
		(self includesClass: each)
			ifTrue: [ aBlock value: each ] ]! !

!RBBrowserEnvironment methodsFor: 'copying' stamp: 'lr 2/26/2009 14:23'!
copyEmpty
	^ self class new! !

!RBBrowserEnvironment methodsFor: 'private' stamp: 'lr 3/18/2011 21:07'!
defaultLabel
	^ 'Smalltalk'! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 08:37'!
definesClass: aClass
	^ true! !

!RBBrowserEnvironment methodsFor: 'private' stamp: 'lr 3/18/2011 21:07'!
environment
	^ self! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:31'!
forCategories: categoryList 
	^RBCategoryEnvironment onEnvironment: self categories: categoryList! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
forClass: aClass protocols: protocolCollection 
	^RBProtocolEnvironment onEnvironment: self
		class: aClass
		protocols: protocolCollection! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
forClass: aClass selectors: selectorCollection 
	^(RBSelectorEnvironment 
		onMethods: selectorCollection
		forClass: aClass
		in: self)
		label: aClass name , '>>' 
					, (selectorCollection detect: [:each | true] ifNone: ['']);
		yourself! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
forClasses: classCollection 
	| classes |
	classes := OrderedCollection new: classCollection size * 2.
	classCollection do: 
			[:each | 
			classes add: each;
				add: each class].
	^RBClassEnvironment onEnvironment: self classes: classes! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
forPackageNames: aCollection
	^ RBPackageEnvironment onEnvironment: self packageNames: aCollection! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
forPackages: aCollection
	^ RBPackageEnvironment onEnvironment: self packages: aCollection! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
forPragmas: aKeywordCollection
	^ RBPragmaEnvironment onEnvironment: self keywords: aKeywordCollection! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
implementorsMatching: aString 
	^RBSelectorEnvironment implementorsMatching: aString in: self! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
implementorsOf: aSelector 
	^RBSelectorEnvironment implementorsOf: aSelector in: self! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 3/18/2011 21:08'!
includesCategory: aCategory
	^ true! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 3/18/2011 21:08'!
includesClass: aClass
	^ true! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 3/18/2011 21:08'!
includesProtocol: aProtocol in: aClass
	^ true! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 3/18/2011 21:08'!
includesSelector: aSelector in: aClass
	^ true! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
instVarReadersTo: instVarName in: aClass 
	^RBVariableEnvironment 
		on: self
		readersOfInstanceVariable: instVarName
		in: aClass! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
instVarRefsTo: instVarName in: aClass 
	^RBVariableEnvironment 
		on: self
		referencesToInstanceVariable: instVarName
		in: aClass! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
instVarWritersTo: instVarName in: aClass 
	^RBVariableEnvironment 
		on: self
		writersOfInstanceVariable: instVarName
		in: aClass! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
instanceVariablesFor: aClass 
	^aClass instVarNames! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 1/3/2010 11:11'!
isClassEnvironment
	^ false! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 3/18/2011 21:08'!
isEmpty
	^ false! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 8/7/2009 13:00'!
isSelectorEnvironment
	^ false! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 3/18/2011 21:08'!
isSystem
	^ true! !

!RBBrowserEnvironment methodsFor: 'testing' stamp: 'lr 1/3/2010 11:11'!
isVariableEnvironment
	^ false! !

!RBBrowserEnvironment methodsFor: 'accessing-classes' stamp: 'lr 3/20/2011 11:18'!
keys
	| keys |
	keys := Set new.
	self systemDictionary keysAndValuesDo: [ :key :value | 
		| class |
		value isBehavior ifTrue: [
			(self includesClass: value) 
				ifTrue: [ keys add: key ] ].
			class := value class.
			(self includesClass: class) 
				ifTrue: [ keys add: key ] ].
	^ keys! !

!RBBrowserEnvironment methodsFor: 'private' stamp: 'lr 3/18/2011 21:07'!
label
	^ label isNil
		ifTrue: [ self defaultLabel ]
		ifFalse: [ label ]! !

!RBBrowserEnvironment methodsFor: 'initialize-release' stamp: ''!
label: aString
	label := aString! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
matches: aString 
	^RBSelectorEnvironment matches: aString in: self! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
not
	self isSystem ifTrue: [^RBSelectorEnvironment new].
	^RBNotEnvironment onEnvironment: self! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
numberClasses
	^self classNames size! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
numberSelectors
	| total |
	total := 0.
	self 
		allClassesDo: [:each | self selectorsForClass: each do: [:sel | total := total + 1]].
	^total! !

!RBBrowserEnvironment methodsFor: 'printing' stamp: ''!
printOn: aStream
	aStream nextPutAll: self label! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
problemCount
	^self numberSelectors! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
protocolsFor: aClass
	^aClass organization categories select: [:each | self includesProtocol: each in: aClass]! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
referencesTo: aLiteral 
	^RBSelectorEnvironment referencesTo: aLiteral in: self! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: ''!
referencesTo: aLiteral in: aClass 
	| classes |
	classes := aClass withAllSuperclasses asSet.
	classes
		addAll: aClass allSubclasses;
		addAll: aClass class withAllSuperclasses;
		addAll: aClass class allSubclasses.
	^(self forClasses: classes) referencesTo: aLiteral! !

!RBBrowserEnvironment methodsFor: 'private' stamp: 'lr 3/20/2011 11:19'!
rootEnvironment
	"The root environment representing everything."

	^ self! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
searchStrings
	^searchStrings isNil
		ifTrue: [#()]
		ifFalse: [searchStrings]! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
searchStrings: aCollection 
	searchStrings := aCollection! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
selectMethods: aBlock 
	| env |
	env := RBSelectorEnvironment onEnvironment: self.
	self classesAndSelectorsDo: 
			[:each :sel | 
			(aBlock value: (each compiledMethodAt: sel)) 
				ifTrue: [env addClass: each selector: sel]].
	^env! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'bh 5/8/2000 21:02'!
selectionIntervalFor: aString 
	| interval |
	self searchStrings isEmpty ifTrue: [^nil].
	interval := self selectionParseTreeIntervalFor: aString.
	interval notNil ifTrue: [^interval].
	self searchStrings do: 
			[:each | 
			| search index |
			search := each isSymbol ifTrue: [each keywords first] ifFalse: [each].
			index := aString indexOfSubCollection: search startingAt: 1.
			index > 0 ifTrue: [^index to: index + search size - 1]].
	^nil! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'lr 9/4/2010 15:45'!
selectionParseTreeIntervalFor: aString 
	| parseTree answerBlock |
	parseTree := RBParser
		parseMethod: aString
		onError: [ :str :pos | ^ nil ].
	answerBlock := [ :aNode :answer | ^ aNode sourceInterval ].
	self searchStrings do: [ :each | 
		| matcher tree |
		matcher := RBParseTreeSearcher new.
		matcher matchesTree: (RBLiteralNode value: each) do: answerBlock.
		each isSymbol 
			ifTrue: [
				tree := RBParseTreeSearcher buildSelectorTree: each.
				tree notNil ifTrue: [ matcher matchesTree: tree do: answerBlock ] ]
			ifFalse: [
				tree := RBVariableNode named: each.
				matcher
					matchesTree: tree do: answerBlock;
					matchesArgumentTree: tree do: answerBlock ].
		matcher executeTree: parseTree ].
	^ nil! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
selectorsFor: aProtocol in: aClass
	^(aClass organization listAtCategoryNamed: aProtocol)
		select: [:each | self includesSelector: each in: aClass]! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'lr 2/26/2009 14:29'!
selectorsForClass: aClass 
	| selectors |
	selectors := IdentitySet new.
	self selectorsForClass: aClass do: [ :each | selectors add: each ].
	^ selectors! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
selectorsForClass: aClass do: aBlock 
	aClass selectorsAndMethodsDo: 
			[:each :meth | 
			(self includesSelector: each in: aClass) ifTrue: [aBlock value: each]]! !

!RBBrowserEnvironment methodsFor: 'printing' stamp: ''!
storeOn: aStream 
	aStream
		nextPutAll: self class name;
		nextPutAll: ' new'! !

!RBBrowserEnvironment methodsFor: 'private' stamp: 'lr 3/20/2011 11:30'!
systemDictionary
	"The root system dictionary as the source of all objects in this environment."

	^ Smalltalk globals! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: 'lr 8/9/2011 21:00'!
whichCategoryIncludes: aClassName
	^ self systemDictionary organization categoryOfElement: aClassName! !

!RBBrowserEnvironment methodsFor: 'accessing' stamp: ''!
whichProtocolIncludes: aSelector in: aClass
	^aClass organization categoryOfElement: aSelector! !

!RBBrowserEnvironment methodsFor: 'environments' stamp: 'lr 9/8/2011 20:32'!
| anEnvironment 
	"If we or anEnvironment includes everything, then return it instead of creating 
	an or that will include everything."

	self isSystem ifTrue: [^self].
	anEnvironment isSystem ifTrue: [^anEnvironment].
	^ RBOrEnvironment onEnvironment: self or: anEnvironment! !

RBBrowserEnvironment subclass: #RBBrowserEnvironmentWrapper
	instanceVariableNames: 'environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

RBBrowserEnvironmentWrapper subclass: #RBAndEnvironment
	instanceVariableNames: 'andedEnvironment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBAndEnvironment class methodsFor: 'instance creation' stamp: ''!
onEnvironment: anEnvironment and: anotherEnvironment 
	^(self onEnvironment: anEnvironment)
		andedEnvironment: anotherEnvironment;
		yourself! !

!RBAndEnvironment methodsFor: 'private' stamp: ''!
andedEnvironment
	^andedEnvironment! !

!RBAndEnvironment methodsFor: 'initialize-release' stamp: ''!
andedEnvironment: aBrowserEnvironment 
	andedEnvironment := aBrowserEnvironment! !

!RBAndEnvironment methodsFor: 'accessing' stamp: ''!
classesDo: aBlock 
	environment 
		classesDo: [:each | (self includesClass: each) ifTrue: [aBlock value: each]]! !

!RBAndEnvironment methodsFor: 'testing' stamp: ''!
includesCategory: aCategory 
	^(self classNamesFor: aCategory) isEmpty not! !

!RBAndEnvironment methodsFor: 'testing' stamp: ''!
includesClass: aClass 
	| doesntHaveSelectors |
	(environment includesClass: aClass) ifFalse: [^false].
	(andedEnvironment includesClass: aClass) ifFalse: [^false].
	doesntHaveSelectors := true.
	environment selectorsForClass: aClass
		do: 
			[:each | 
			doesntHaveSelectors := false.
			(andedEnvironment includesSelector: each in: aClass) ifTrue: [^true]].
	^doesntHaveSelectors! !

!RBAndEnvironment methodsFor: 'testing' stamp: ''!
includesProtocol: aProtocol in: aClass 
	^(self selectorsFor: aProtocol in: aClass) isEmpty not! !

!RBAndEnvironment methodsFor: 'testing' stamp: ''!
includesSelector: aSelector in: aClass 
	^(environment includesSelector: aSelector in: aClass) 
		and: [andedEnvironment includesSelector: aSelector in: aClass]! !

!RBAndEnvironment methodsFor: 'accessing' stamp: ''!
numberSelectors
	| total |
	total := 0.
	environment classesAndSelectorsDo: 
			[:each :sel | 
			(andedEnvironment includesSelector: sel in: each) 
				ifTrue: [total := total + 1]].
	^total! !

!RBAndEnvironment methodsFor: 'accessing' stamp: ''!
problemCount
	^environment isClassEnvironment 
		ifTrue: [self numberClasses]
		ifFalse: [super problemCount]! !

!RBAndEnvironment methodsFor: 'accessing' stamp: 'bh 5/8/2000 21:01'!
selectionIntervalFor: aString 
	| interval |
	interval := super selectionIntervalFor: aString.
	interval notNil ifTrue: [^interval].
	^andedEnvironment selectionIntervalFor: aString
! !

!RBAndEnvironment methodsFor: 'accessing' stamp: ''!
selectorsForClass: aClass do: aBlock 
	environment selectorsForClass: aClass
		do: 
			[:each | 
			(andedEnvironment includesSelector: each in: aClass) 
				ifTrue: [aBlock value: each]]! !

!RBAndEnvironment methodsFor: 'printing' stamp: ''!
storeOn: aStream 
	aStream nextPut: $(.
	environment storeOn: aStream.
	aStream nextPutAll: ' & '.
	andedEnvironment storeOn: aStream.
	aStream nextPut: $)! !

!RBBrowserEnvironmentWrapper class methodsFor: 'instance creation' stamp: 'lr 9/8/2011 20:25'!
new
	^ self onEnvironment: RBBrowserEnvironment new! !

!RBBrowserEnvironmentWrapper class methodsFor: 'instance creation' stamp: ''!
onEnvironment: anEnvironment 
	^(self basicNew)
		initialize;
		onEnvironment: anEnvironment;
		yourself! !

!RBBrowserEnvironmentWrapper methodsFor: 'testing' stamp: 'lr 11/25/2009 08:38'!
definesClass: aClass
	^ environment definesClass: aClass! !

!RBBrowserEnvironmentWrapper methodsFor: 'private' stamp: 'lr 3/18/2011 21:07'!
environment
	^ environment! !

!RBBrowserEnvironmentWrapper methodsFor: 'testing' stamp: ''!
includesCategory: aCategory
	^environment includesCategory: aCategory! !

!RBBrowserEnvironmentWrapper methodsFor: 'testing' stamp: ''!
includesClass: aClass
	^environment includesClass: aClass! !

!RBBrowserEnvironmentWrapper methodsFor: 'testing' stamp: ''!
includesProtocol: aProtocol in: aClass
	^(self includesClass: aClass)
		and: [environment includesProtocol: aProtocol in: aClass]! !

!RBBrowserEnvironmentWrapper methodsFor: 'testing' stamp: ''!
includesSelector: aSelector in: aClass
	^(self includesClass: aClass)
		and: [environment includesSelector: aSelector in: aClass]! !

!RBBrowserEnvironmentWrapper methodsFor: 'testing' stamp: ''!
isEmpty
	self classesDo: [:each | ^false].
	^true! !

!RBBrowserEnvironmentWrapper methodsFor: 'testing' stamp: ''!
isSystem
	^false! !

!RBBrowserEnvironmentWrapper methodsFor: 'initialize-release' stamp: ''!
onEnvironment: anEnvironment
	environment := anEnvironment! !

!RBBrowserEnvironmentWrapper methodsFor: 'private' stamp: 'lr 3/20/2011 11:17'!
rootEnvironment
	^ environment rootEnvironment! !

!RBBrowserEnvironmentWrapper methodsFor: 'accessing' stamp: 'bh 5/8/2000 21:03'!
selectionIntervalFor: aString 
	| interval |
	interval := super selectionIntervalFor: aString.
	^interval notNil 
		ifTrue: [interval]
		ifFalse: [environment selectionIntervalFor: aString]! !

!RBBrowserEnvironmentWrapper methodsFor: 'printing' stamp: ''!
storeOn: aStream 
	aStream
		nextPutAll: '(';
		nextPutAll: self class name;
		nextPutAll: ' onEnvironment: '.
	environment storeOn: aStream.
	aStream nextPut: $)! !

!RBBrowserEnvironmentWrapper methodsFor: 'private' stamp: 'lr 3/20/2011 11:18'!
systemDictionary
	^ environment systemDictionary! !

RBBrowserEnvironmentWrapper subclass: #RBCategoryEnvironment
	instanceVariableNames: 'categories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBCategoryEnvironment class methodsFor: 'instance creation' stamp: ''!
onEnvironment: anEnvironment categories: aCollection
	^(self onEnvironment: anEnvironment)
		categories: aCollection; yourself! !

!RBCategoryEnvironment methodsFor: 'adding' stamp: 'lr 2/8/2009 10:46'!
addCategory: aSymbol
	categories add: aSymbol! !

!RBCategoryEnvironment methodsFor: 'initialize-release' stamp: 'lr 2/26/2009 14:25'!
categories: aCollection
	categories addAll: aCollection! !

!RBCategoryEnvironment methodsFor: 'accessing-classes' stamp: 'lr 2/26/2009 14:32'!
classNames
	^ self categories 
		inject: IdentitySet new
		into: [ :answer :each | 
			answer
				addAll: (self classNamesFor: each);
				yourself ]! !

!RBCategoryEnvironment methodsFor: 'private' stamp: ''!
defaultLabel
	| stream |
	stream := String new writeStream.
	categories do: [:each | stream nextPutAll: each; nextPut: $ ].
	^stream contents! !

!RBCategoryEnvironment methodsFor: 'testing' stamp: ''!
includesCategory: aCategory
	^(categories includes: aCategory)
		and: [super includesCategory: aCategory]! !

!RBCategoryEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 08:31'!
includesClass: aClass 
	^ (super includesClass: aClass) and: [ categories includes: aClass theNonMetaClass category ]! !

!RBCategoryEnvironment methodsFor: 'initialize-release' stamp: 'lr 2/26/2009 14:25'!
initialize
	super initialize.
	categories := IdentitySet new! !

!RBCategoryEnvironment methodsFor: 'testing' stamp: ''!
isEmpty
	^categories isEmpty! !

!RBCategoryEnvironment methodsFor: 'accessing' stamp: ''!
numberSelectors
	| total |
	total := 0.
	self 
		classesDo: [:each | self selectorsForClass: each do: [:sel | total := total + 1]].
	^total! !

!RBCategoryEnvironment methodsFor: 'copying' stamp: 'lr 2/26/2009 14:21'!
postCopy
	super postCopy.
	categories := categories copy! !

!RBCategoryEnvironment methodsFor: 'printing' stamp: ''!
storeOn: aStream
	aStream nextPut: $(.
	super storeOn: aStream.
	aStream nextPutAll: ' categories: '.
	categories asArray storeOn: aStream.
	aStream nextPut: $)! !

RBBrowserEnvironmentWrapper subclass: #RBClassEnvironment
	instanceVariableNames: 'classes metaClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBClassEnvironment class methodsFor: 'instance creation' stamp: ''!
onEnvironment: anEnvironment classes: aCollection
	^(self onEnvironment: anEnvironment)
		classes: aCollection; yourself! !

!RBClassEnvironment methodsFor: 'adding' stamp: 'lr 9/14/2010 13:07'!
addClass: aClass 
	aClass isMeta
		ifTrue: [ metaClasses add: aClass soleInstance name ]
		ifFalse: [ classes add: aClass name ]! !

!RBClassEnvironment methodsFor: 'accessing' stamp: 'lr 9/8/2011 20:32'!
asSelectorEnvironment

	^RBSelectorEnvironment new
		searchStrings:#();
		label:self label;
		onEnvironment: self environment;
		classSelectors: self classSelectorDictionary metaClassSelectors: self metaClassSelectorDictionary;
		yourself.! !

!RBClassEnvironment methodsFor: 'accessing-classes' stamp: 'lr 2/26/2009 14:24'!
classNames
	^ IdentitySet new
		addAll: classes;
		addAll: metaClasses;
		yourself! !

!RBClassEnvironment methodsFor: 'printing' stamp: 'lr 3/20/2011 11:18'!
classSelectorDictionary
	^ classes 
		inject: (IdentityDictionary new: classes size)
		into: [ :answer :class |
			answer
				at: class put: (self systemDictionary at: class) selectors;
				yourself ]! !

!RBClassEnvironment methodsFor: 'initialize-release' stamp: 'lr 9/14/2010 13:06'!
classes: aCollection 
	aCollection do: [ :each | self addClass: each ]! !

!RBClassEnvironment methodsFor: 'accessing-classes' stamp: 'lr 3/20/2011 11:18'!
classesDo: aBlock 
	classes do: [ :each | 
		| class |
		class := self systemDictionary at: each ifAbsent: [ nil ].
		(class notNil and: [ environment includesClass: class ])
			ifTrue: [ aBlock value: class ] ].
	metaClasses do: [ :each | 
		| class |
		class := self systemDictionary at: each ifAbsent: [ nil ].
		(class notNil and: [ environment includesClass: class class ])
			ifTrue: [ aBlock value: class class ] ]! !

!RBClassEnvironment methodsFor: 'private' stamp: ''!
defaultLabel
	| stream |
	stream := String new writeStream.
	classes do: 
			[:each | 
			stream nextPutAll: each;
				nextPut: $ ].
	^stream contents! !

!RBClassEnvironment methodsFor: 'testing' stamp: 'lr 3/20/2011 11:18'!
includesCategory: aCategory
	^ (super includesCategory: aCategory) and: [
		(environment classNamesFor: aCategory)
			inject: false
			into: [ :bool :each |
				bool or: [
					| class |
					class := self systemDictionary at: each ifAbsent: [ nil ].
					class notNil 
						and: [ (self includesClass: class)
						or: [ self includesClass: class class ] ] ] ] ]! !

!RBClassEnvironment methodsFor: 'testing' stamp: ''!
includesClass: aClass 
	^(aClass isMeta
		ifTrue: [metaClasses includes: aClass soleInstance name]
		ifFalse: [classes includes: aClass name])
			and: [super includesClass: aClass]! !

!RBClassEnvironment methodsFor: 'initialize-release' stamp: 'lr 2/26/2009 13:35'!
initialize
	super initialize.
	classes := IdentitySet new.
	metaClasses := IdentitySet new! !

!RBClassEnvironment methodsFor: 'testing' stamp: ''!
isClassEnvironment
	^true! !

!RBClassEnvironment methodsFor: 'testing' stamp: ''!
isEmpty
	^classes isEmpty and: [metaClasses isEmpty]! !

!RBClassEnvironment methodsFor: 'printing' stamp: 'lr 3/20/2011 11:18'!
metaClassSelectorDictionary
	^ metaClasses 
		inject: (IdentityDictionary new: metaClasses size)
		into: [ :answer :class |
			answer
				at: class put: (self systemDictionary at: class) class selectors;
				yourself ]! !

!RBClassEnvironment methodsFor: 'copying' stamp: 'lr 2/26/2009 14:24'!
postCopy
	super postCopy.
	classes := classes copy.
	metaClasses := metaClasses copy! !

!RBClassEnvironment methodsFor: 'accessing' stamp: ''!
problemCount
	^self numberClasses! !

!RBClassEnvironment methodsFor: 'removing' stamp: ''!
removeClass: aClass 
	aClass isMeta
		ifTrue: [metaClasses remove: aClass soleInstance name ifAbsent: []]
		ifFalse: [classes remove: aClass name ifAbsent: []]! !

!RBClassEnvironment methodsFor: 'printing' stamp: ''!
storeOn: aStream
	aStream nextPut: $(.
	super storeOn: aStream.
	aStream nextPutAll: ' classes: (('.
	classes asArray storeOn: aStream.
	aStream nextPutAll: ' inject: OrderedCollection new into: [:sum :each | 
				| class |
				class := Smalltalk at: each ifAbsent: [nil].
				class notNil ifTrue: [sum add: class].
				sum]) , ('.
	metaClasses asArray storeOn: aStream.
	aStream nextPutAll: ' inject: OrderedCollection new into: [:sum :each | 
				| class |
				class := Smalltalk at: each ifAbsent: [nil].
				class notNil ifTrue: [sum add: class class].
				sum])))'! !

RBBrowserEnvironmentWrapper subclass: #RBNotEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBNotEnvironment methodsFor: 'testing' stamp: ''!
includesCategory: aCategory 
	^(self classNamesFor: aCategory) isEmpty not! !

!RBNotEnvironment methodsFor: 'testing' stamp: ''!
includesClass: aClass 
	(environment includesClass: aClass) ifFalse: [^true].
	aClass selectorsAndMethodsDo: 
			[:each :meth | 
			(environment includesSelector: each in: aClass) ifFalse: [^true]].
	^false! !

!RBNotEnvironment methodsFor: 'testing' stamp: ''!
includesProtocol: aProtocol in: aClass 
	^(self selectorsFor: aProtocol in: aClass) isEmpty not! !

!RBNotEnvironment methodsFor: 'testing' stamp: ''!
includesSelector: aSelector in: aClass
	^(environment includesSelector: aSelector in: aClass) not! !

!RBNotEnvironment methodsFor: 'environments' stamp: ''!
not
	^environment! !

!RBNotEnvironment methodsFor: 'printing' stamp: ''!
storeOn: aStream
	environment storeOn: aStream.
	aStream nextPutAll: ' not'! !

RBBrowserEnvironmentWrapper subclass: #RBOrEnvironment
	instanceVariableNames: 'orEnvironment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBOrEnvironment class methodsFor: 'as yet unclassified' stamp: 'lr 11/25/2009 00:51'!
onEnvironment: anEnvironment or: anotherEnvironment
	^ (self onEnvironment: anEnvironment)
		orEnvironment: anotherEnvironment;
		yourself! !

!RBOrEnvironment methodsFor: 'accessing' stamp: 'lr 11/25/2009 08:25'!
classesDo: aBlock
	| enumerated |
	enumerated := IdentitySet new.
	environment classesDo: [ :each | 
		aBlock value: each.
		enumerated add: each].
	orEnvironment classesDo: [ :each | 
		(enumerated includes: each) 
			ifFalse: [ aBlock value: each ] ]! !

!RBOrEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 08:26'!
includesCategory: aCategory 
	^ (environment includesCategory: aCategory)
		or: [ orEnvironment includesCategory: aCategory ]! !

!RBOrEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 00:53'!
includesClass: aClass
	(environment includesClass: aClass) ifTrue: [ ^ true ].
	(orEnvironment includesClass: aClass) ifTrue: [ ^ true ].
	(environment selectorsForClass: aClass) isEmpty ifFalse: [ ^ true ].
	(orEnvironment selectorsForClass: aClass) isEmpty ifFalse: [ ^ true ].
	^ false! !

!RBOrEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 00:53'!
includesProtocol: aProtocol in: aClass
	^ (environment includesProtocol: aProtocol in: aClass)
		or: [ orEnvironment includesProtocol: aProtocol in: aClass ]! !

!RBOrEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 00:53'!
includesSelector: aSelector in: aClass
	^ (environment includesSelector: aSelector in: aClass)
		or: [ orEnvironment includesSelector: aSelector in: aClass ]! !

!RBOrEnvironment methodsFor: 'accessing' stamp: 'lr 11/25/2009 08:25'!
numberSelectors
	| total |
	total := 0.
	self classesDo: [ :each | 
		self
			selectorsForClass: each
			do: [ :selector | total := total + 1 ] ].
	^ total! !

!RBOrEnvironment methodsFor: 'initialization' stamp: 'lr 11/25/2009 00:49'!
orEnvironment: aBrowserEnvironment 
	orEnvironment := aBrowserEnvironment! !

!RBOrEnvironment methodsFor: 'accessing' stamp: 'lr 11/25/2009 00:54'!
selectionIntervalFor: aString
	^ (environment selectionIntervalFor: aString)
		ifNil: [ orEnvironment selectionIntervalFor: aString ]! !

!RBOrEnvironment methodsFor: 'accessing' stamp: 'lr 11/25/2009 08:26'!
selectorsForClass: aClass do: aBlock
	| enumerated |
	enumerated := IdentitySet new.
	environment
		selectorsForClass: aClass
		do: [ :each | 
			enumerated add: each.
			aBlock value: each ].
	orEnvironment
		selectorsForClass: aClass
		do: [:each | 
			(enumerated includes: each) 
				ifFalse: [ aBlock value: each ] ]! !

!RBOrEnvironment methodsFor: 'printing' stamp: 'lr 11/25/2009 00:51'!
storeOn: aStream 
	aStream nextPut: $(.
	environment storeOn: aStream.
	aStream nextPutAll: ' | '.
	orEnvironment storeOn: aStream.
	aStream nextPut: $)! !

RBBrowserEnvironmentWrapper subclass: #RBPackageEnvironment
	instanceVariableNames: 'packages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBPackageEnvironment class methodsFor: 'instance creation' stamp: 'lr 11/25/2009 08:54'!
onEnvironment: anEnvironment packageNames: aCollection
	^ self onEnvironment: anEnvironment packages: (aCollection collect: [ :each | PackageInfo named: each ])! !

!RBPackageEnvironment class methodsFor: 'instance creation' stamp: 'lr 11/25/2009 08:54'!
onEnvironment: anEnvironment packages: aCollection
	^ (self onEnvironment: anEnvironment)
		packages: aCollection;
		yourself! !

!RBPackageEnvironment methodsFor: 'accessing' stamp: 'lr 9/8/2011 20:32'!
asSelectorEnvironment
	| result |
	result := RBSelectorEnvironment onEnvironment: environment.
	self classesAndSelectorsDo: [ :class :selector | result addClass: class selector: selector ].
	^ result! !

!RBPackageEnvironment methodsFor: 'accessing' stamp: 'lr 11/25/2009 08:48'!
classesAndSelectorsDo: aBlock
	packages do: [ :package |
		package methods do: [ :method |
			(environment includesSelector: method methodSymbol in: method actualClass)
				ifTrue: [ aBlock value: method actualClass value: method methodSymbol ] ] ]! !

!RBPackageEnvironment methodsFor: 'accessing' stamp: 'lr 12/3/2009 10:16'!
classesDo: aBlock
	| enumerated enumerator |
	enumerated := IdentitySet new.
	enumerator := [ :each |
		(enumerated includes: each) ifFalse: [
			(environment includesClass: each)
				ifTrue: [ aBlock value: each ].
			(environment includesClass: each class)	
				ifTrue: [ aBlock value: each class ].
			enumerated add: each ] ].
	packages do: [ :package |
		package classes do: enumerator.
		package extensionClasses do: enumerator ]! !

!RBPackageEnvironment methodsFor: 'testing' stamp: 'lr 5/15/2010 09:39'!
definesClass: aClass
	^ (super definesClass: aClass) and: [ self packages anySatisfy: [ :package | package includesClass: aClass ] ]! !

!RBPackageEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 08:51'!
includesCategory: aCategory
	^ (super includesCategory: aCategory) and: [ self packages anySatisfy: [ :package | package includesSystemCategory: aCategory ] ]! !

!RBPackageEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 09:06'!
includesClass: aClass
	^ (super includesClass: aClass) and: [ self packages anySatisfy: [ :package | (package includesClass: aClass) or: [ (package extensionCategoriesForClass: aClass) notEmpty ] ] ]! !

!RBPackageEnvironment methodsFor: 'testing' stamp: 'lr 12/3/2009 10:13'!
includesProtocol: aProtocol in: aClass
	^ (environment includesProtocol: aProtocol in: aClass) and: [ self packages anySatisfy: [ :package | package includesMethodCategory: aProtocol ofClass: aClass ] ]! !

!RBPackageEnvironment methodsFor: 'testing' stamp: 'lr 11/25/2009 10:33'!
includesSelector: aSelector in: aClass
	^ (environment includesSelector: aSelector in: aClass) and: [ self packages anySatisfy: [ :package | package includesMethod: aSelector ofClass: aClass ] ]! !

!RBPackageEnvironment methodsFor: 'initialize-release' stamp: 'lr 4/12/2010 15:25'!
initialize
	super initialize.
	packages := Set new! !

!RBPackageEnvironment methodsFor: 'accessing' stamp: 'lr 12/3/2009 13:30'!
numberSelectors
	^ packages inject: 0 into: [ :result :package | result + package methods size ]! !

!RBPackageEnvironment methodsFor: 'accessing' stamp: 'lr 11/25/2009 08:49'!
packageNames
	^ packages collect: [ :each | each packageName ]! !

!RBPackageEnvironment methodsFor: 'accessing' stamp: 'lr 11/25/2009 08:46'!
packages
	^ packages! !

!RBPackageEnvironment methodsFor: 'initialize-release' stamp: 'lr 12/20/2009 09:29'!
packages: aCollection
	packages addAll: aCollection! !

!RBPackageEnvironment methodsFor: 'copying' stamp: 'lr 4/12/2010 15:26'!
postCopy
	super postCopy.
	packages := packages copy! !

!RBPackageEnvironment methodsFor: 'printing' stamp: 'TestRunner 12/23/2009 21:22'!
storeOn: aStream 
	aStream nextPut: $(; nextPutAll: self class name; nextPutAll: ' onEnvironment: '.
	environment storeOn: aStream.
	aStream nextPutAll: ' packageNames: '.
	self packageNames asArray storeOn: aStream.
	aStream nextPut: $)! !

RBBrowserEnvironmentWrapper subclass: #RBPragmaEnvironment
	instanceVariableNames: 'keywords condition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBPragmaEnvironment class methodsFor: 'instance creation' stamp: 'lr 7/21/2008 10:38'!
onEnvironment: anEnvironment keywords: aKeywordCollection
	^ (self onEnvironment: anEnvironment)
		keywords: aKeywordCollection;
		yourself! !

!RBPragmaEnvironment methodsFor: 'adding' stamp: 'lr 7/21/2008 10:20'!
addKeyword: aSymbol
	keywords add: aSymbol! !

!RBPragmaEnvironment methodsFor: 'initialize-release' stamp: 'lr 7/21/2008 10:34'!
condition: aBlock
	condition := aBlock! !

!RBPragmaEnvironment methodsFor: 'private' stamp: 'lr 7/21/2008 10:40'!
defaultLabel
	| stream |
	stream := String new writeStream.
	keywords do: [ :each | stream nextPut: $<; nextPutAll: each; nextPut: $>; nextPut: $ ].
	^ stream contents! !

!RBPragmaEnvironment methodsFor: 'testing' stamp: 'lr 8/7/2009 12:42'!
includesCategory: aCategory
	^ (environment includesCategory: aCategory) and: [ (self classNamesFor: aCategory) notEmpty ]! !

!RBPragmaEnvironment methodsFor: 'testing' stamp: 'lr 8/7/2009 10:43'!
includesClass: aClass
	^ (environment includesClass: aClass) and: [ aClass selectors anySatisfy: [ :each | self includesSelector: each in: aClass ] ]! !

!RBPragmaEnvironment methodsFor: 'testing' stamp: 'lr 7/21/2008 10:34'!
includesPragma: aPragma
	^ (keywords includes: aPragma keyword) and: [ condition value: aPragma ]! !

!RBPragmaEnvironment methodsFor: 'testing' stamp: 'lr 8/7/2009 12:42'!
includesProtocol: aProtocol in: aClass
	^ (environment includesProtocol: aProtocol in: aClass) and: [ (self selectorsFor: aProtocol in: aClass) notEmpty ]! !

!RBPragmaEnvironment methodsFor: 'testing' stamp: 'lr 1/3/2010 11:48'!
includesSelector: aSelector in: aClass
	(environment includesSelector: aSelector in: aClass)
		ifFalse: [ ^ false ].
	^ (aClass compiledMethodAt: aSelector) pragmas
		anySatisfy: [ :each | self includesPragma: each ]! !

!RBPragmaEnvironment methodsFor: 'initialize-release' stamp: 'lr 2/26/2009 13:35'!
initialize
	super initialize.
	keywords := IdentitySet new.
	condition := [ :pragma | true ]! !

!RBPragmaEnvironment methodsFor: 'initialize-release' stamp: 'lr 2/24/2009 19:38'!
keywords: aCollection
	keywords addAll: aCollection! !

!RBPragmaEnvironment methodsFor: 'copying' stamp: 'lr 7/21/2008 10:37'!
postCopy
	super postCopy.
	keywords := keywords copy! !

!RBPragmaEnvironment methodsFor: 'printing' stamp: 'lr 7/21/2008 10:37'!
storeOn: aStream
	aStream nextPut: $(.
	super storeOn: aStream.
	aStream nextPutAll: ' keywords: '.
	keywords asArray storeOn: aStream.
	aStream nextPut: $)! !

RBBrowserEnvironmentWrapper subclass: #RBProtocolEnvironment
	instanceVariableNames: 'class protocols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBProtocolEnvironment class methodsFor: 'instance creation' stamp: ''!
onEnvironment: anEnvironment class: aClass protocols: aCollection
	^(self onEnvironment: anEnvironment)
		class: aClass protocols: aCollection; yourself! !

!RBProtocolEnvironment methodsFor: 'initialize-release' stamp: ''!
class: aClass protocols: aCollection
	class := aClass.
	protocols := aCollection! !

!RBProtocolEnvironment methodsFor: 'private' stamp: ''!
defaultLabel
	| stream |
	stream := String new writeStream.
	stream nextPutAll: class name;
		nextPut: $>.
	protocols do: 
			[:each | 
			stream nextPutAll: each;
				nextPut: $ ].
	^stream contents! !

!RBProtocolEnvironment methodsFor: 'testing' stamp: 'lr 3/20/2011 11:18'!
includesCategory: aCategory
	^ (super includesCategory: aCategory) and: [
		(environment classNamesFor: aCategory)
			inject: false into: [ :bool :each |
				bool or: [
					| aClass |
					aClass := self systemDictionary at: each ifAbsent: [ nil ].
					aClass == class or: [ aClass class == class ] ] ] ]! !

!RBProtocolEnvironment methodsFor: 'testing' stamp: ''!
includesClass: aClass
	^aClass == class and: [super includesClass: aClass]! !

!RBProtocolEnvironment methodsFor: 'testing' stamp: ''!
includesProtocol: aProtocol in: aClass
	^aClass == class and: [(super includesProtocol: aProtocol in: aClass)
			and: [protocols includes: aProtocol]]! !

!RBProtocolEnvironment methodsFor: 'testing' stamp: ''!
includesSelector: aSelector in: aClass 
	^(super includesSelector: aSelector in: aClass) and: 
			[protocols 
				includes: (environment whichProtocolIncludes: aSelector in: aClass)]! !

!RBProtocolEnvironment methodsFor: 'testing' stamp: ''!
isEmpty
	^protocols isEmpty! !

!RBProtocolEnvironment methodsFor: 'copying' stamp: 'lr 2/26/2009 14:29'!
postCopy
	super postCopy.
	protocols := protocols copy! !

!RBProtocolEnvironment methodsFor: 'printing' stamp: ''!
storeOn: aStream 
	aStream nextPut: $(.
	super storeOn: aStream.
	aStream
		nextPutAll: ' class: ';
		nextPutAll: class name;
		nextPutAll: ' protocols: '.
	protocols asArray storeOn: aStream.
	aStream nextPut: $)! !

RBBrowserEnvironmentWrapper subclass: #RBSelectorEnvironment
	instanceVariableNames: 'classSelectors metaClassSelectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBSelectorEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/14/2010 11:59'!
implementorsMatching: aString in: anEnvironment 
	| classDict metaDict |
	classDict := IdentityDictionary new.
	metaDict := IdentityDictionary new.
	anEnvironment classesDo: [ :class | 
		| selectors |
		selectors := IdentitySet new.
		anEnvironment selectorsForClass: class
			do: [ :each | (aString match: each) ifTrue: [ selectors add: each ] ].
		selectors isEmpty ifFalse: [
			class isMeta 
				ifTrue: [ metaDict at: class soleInstance name put: selectors ]
				ifFalse: [ classDict at: class name put: selectors ] ] ].
	^ (self onEnvironment: anEnvironment)
		classSelectors: classDict metaClassSelectors: metaDict;
		label: 'Implementors of ' , aString;
		yourself! !

!RBSelectorEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/14/2010 13:05'!
implementorsOf: aSelector in: anEnvironment 
	| classDict metaDict selectors |
	classDict := IdentityDictionary new.
	metaDict := IdentityDictionary new.
	selectors := IdentitySet with: aSelector.
	anEnvironment classesDo: [ :class | 
		((class includesSelector: aSelector) and: [ anEnvironment includesSelector: aSelector in: class ]) 
			ifTrue: [
				class isMeta 
					ifTrue: [ metaDict at: class soleInstance name put: selectors copy ]
					ifFalse: [ classDict at: class name put: selectors copy ] ] ].
	^ (self onEnvironment: anEnvironment)
		classSelectors: classDict metaClassSelectors: metaDict;
		label: 'Implementors of ' , aSelector;
		yourself! !

!RBSelectorEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/14/2010 12:01'!
matches: aString in: anEnvironment 
	| newEnvironment |
	newEnvironment := (self onEnvironment: anEnvironment)
		label: 'Matching: ' , aString;
		searchStrings: (Array with: aString);
		yourself.
	anEnvironment classesAndSelectorsDo: [ :class :selector | 
		| method |
		method := class compiledMethodAt: selector.
		method allLiterals do: [ :literal | 
			literal isString ifTrue: [
				(aString match: literal)
					ifTrue: [ newEnvironment addClass: class selector: selector ] ] ] ].
	^ newEnvironment! !

!RBSelectorEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/14/2010 12:06'!
onMethods: selectorCollection forClass: aClass in: anEnvironment 
	| environemnt |
	environemnt := self onEnvironment: anEnvironment.
	selectorCollection do: [ :each | environemnt addClass: aClass selector: each ].
	^ environemnt! !

!RBSelectorEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/14/2010 12:07'!
referencesTo: aLiteral in: anEnvironment 
	| classDict literalPrintString specialFlag specialByte |
	literalPrintString := aLiteral isVariableBinding
		ifTrue: [ aLiteral key asString ]
		ifFalse: [
			aLiteral isString
				ifTrue: [ aLiteral ]
				ifFalse: [ aLiteral printString ] ].
	classDict := IdentityDictionary new.
	specialFlag := Smalltalk
		hasSpecialSelector: aLiteral
		ifTrueSetByte: [ :byte | specialByte := byte ].
	anEnvironment classesDo: [ :class | 
		| selectors |
		selectors := (class thoroughWhichSelectorsReferTo: aLiteral special: specialFlag byte: specialByte)
			select: [ :selector | anEnvironment includesSelector: selector in: class ].
		selectors isEmpty 
			ifFalse: [ classDict at: class put: selectors asIdentitySet ] ].
	^ (self onEnvironment: anEnvironment)
		on: classDict;
		label: 'References to: ' , literalPrintString;
		searchStrings: (Array with: literalPrintString);
		yourself! !

!RBSelectorEnvironment methodsFor: 'adding' stamp: 'lr 9/14/2010 11:40'!
addClass: aClass 
	aClass isMeta 
		ifTrue: [ metaClassSelectors at: aClass soleInstance name put: aClass selectors asIdentitySet ]
		ifFalse: [ classSelectors at: aClass name put: aClass selectors asIdentitySet ]! !

!RBSelectorEnvironment methodsFor: 'adding' stamp: 'lr 2/26/2009 13:36'!
addClass: aClass selector: aSymbol 
	(aClass isMeta 
		ifTrue: [ metaClassSelectors at: aClass soleInstance name ifAbsentPut: [ IdentitySet new ] ]
		ifFalse: [ classSelectors at: aClass name ifAbsentPut: [ IdentitySet new ] ])
			add: aSymbol! !

!RBSelectorEnvironment methodsFor: 'accessing' stamp: 'rr 4/19/2004 16:06'!
asSelectorEnvironment
	^ self! !

!RBSelectorEnvironment methodsFor: 'accessing-classes' stamp: 'lr 2/26/2009 13:35'!
classNames
	^ IdentitySet new
		addAll: classSelectors keys;
		addAll: metaClassSelectors keys;
		yourself! !

!RBSelectorEnvironment methodsFor: 'initialize-release' stamp: ''!
classSelectors: classSelectorDictionary metaClassSelectors: metaClassSelectorDictionary 
	classSelectors := classSelectorDictionary.
	metaClassSelectors := metaClassSelectorDictionary! !

!RBSelectorEnvironment methodsFor: 'initialize-release' stamp: 'lr 2/26/2009 13:36'!
classes: classArray metaClasses: metaArray 
	"Used to recreate an environment from its storeString"

	classSelectors := IdentityDictionary new.
	metaClassSelectors := IdentityDictionary new.
	classArray 
		do: [ :each | classSelectors at: each first put: each last asIdentitySet ].
	metaArray 
		do: [ :each | metaClassSelectors at: each first put: each last asIdentitySet ]! !

!RBSelectorEnvironment methodsFor: 'accessing-classes' stamp: 'lr 3/20/2011 11:18'!
classesDo: aBlock 
	classSelectors keysDo: [ :each | 
		| class |
		class := self systemDictionary at: each ifAbsent: [ nil ].
		(class notNil and: [ environment includesClass: class ])
			ifTrue: [ aBlock value: class ] ].
	metaClassSelectors keysDo: [ :each | 
		| class |
		class := self systemDictionary at: each ifAbsent: [ nil ].
		(class notNil and: [ environment includesClass: class class ])
			ifTrue: [ aBlock value: class class ] ]! !

!RBSelectorEnvironment methodsFor: 'private' stamp: ''!
defaultLabel
	^'some methods'! !

!RBSelectorEnvironment methodsFor: 'testing' stamp: 'TestRunner 1/3/2010 12:36'!
includesCategory: aCategory 
	^(super includesCategory: aCategory) and: 
			[(self classNamesFor: aCategory) anySatisfy: 
					[:className | 
					(classSelectors includesKey: className) 
						or: [metaClassSelectors includesKey: className]]]! !

!RBSelectorEnvironment methodsFor: 'testing' stamp: ''!
includesClass: aClass 
	^(self privateSelectorsForClass: aClass) isEmpty not 
		and: [super includesClass: aClass]! !

!RBSelectorEnvironment methodsFor: 'testing' stamp: 'TestRunner 1/3/2010 12:36'!
includesProtocol: aProtocol in: aClass 
	^(super includesProtocol: aProtocol in: aClass)
		and: [(environment selectorsFor: aProtocol in: aClass)
				anySatisfy: [:aSelector | self privateIncludesSelector: aSelector inClass: aClass]]! !

!RBSelectorEnvironment methodsFor: 'testing' stamp: ''!
includesSelector: aSelector in: aClass 
	^(environment includesSelector: aSelector in: aClass)
		and: [self privateIncludesSelector: aSelector inClass: aClass]! !

!RBSelectorEnvironment methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	classSelectors := IdentityDictionary new.
	metaClassSelectors := IdentityDictionary new! !

!RBSelectorEnvironment methodsFor: 'testing' stamp: ''!
isEmpty
	^classSelectors isEmpty and: [metaClassSelectors isEmpty]! !

!RBSelectorEnvironment methodsFor: 'testing' stamp: 'lr 8/7/2009 13:00'!
isSelectorEnvironment
	^ true! !

!RBSelectorEnvironment methodsFor: 'accessing' stamp: ''!
numberSelectors
	"This doesn't compute the correct result when a method that is included in our method list is not in the
	environment we are wrapping. It is implemented this way for efficiency."

	^(classSelectors inject: 0 into: [:sum :each | sum + each size]) 
		+ (metaClassSelectors inject: 0 into: [:sum :each | sum + each size])! !

!RBSelectorEnvironment methodsFor: 'initialize-release' stamp: 'lr 9/14/2010 11:40'!
on: aDictionary 
	aDictionary keysAndValuesDo: [ :class :selectors | 
		class isMeta 
			ifTrue: [ metaClassSelectors at: class soleInstance name put: selectors asIdentitySet ]
			ifFalse: [ classSelectors at: class name put: selectors asIdentitySet ] ]! !

!RBSelectorEnvironment methodsFor: 'copying' stamp: 'lr 2/26/2009 14:29'!
postCopy
	| newDict |
	super postCopy.
	newDict := classSelectors copy.
	newDict keysAndValuesDo: [:key :value | newDict at: key put: value copy].
	classSelectors := newDict.
	newDict := metaClassSelectors copy.
	newDict keysAndValuesDo: [:key :value | newDict at: key put: value copy].
	metaClassSelectors := newDict! !

!RBSelectorEnvironment methodsFor: 'private' stamp: ''!
privateIncludesSelector: aSelector inClass: aClass 
	^(self privateSelectorsForClass: aClass) includes: aSelector! !

!RBSelectorEnvironment methodsFor: 'private' stamp: ''!
privateSelectorsForClass: aClass 
	^aClass isMeta 
		ifTrue: [metaClassSelectors at: aClass soleInstance name ifAbsent: [#()]]
		ifFalse: [classSelectors at: aClass name ifAbsent: [#()]]! !

!RBSelectorEnvironment methodsFor: 'removing' stamp: ''!
removeClass: aClass 
	aClass isMeta 
		ifTrue: [metaClassSelectors removeKey: aClass soleInstance name ifAbsent: []]
		ifFalse: [classSelectors removeKey: aClass name ifAbsent: []]! !

!RBSelectorEnvironment methodsFor: 'removing' stamp: ''!
removeClass: aClass selector: aSelector 
	(aClass isMeta 
		ifTrue: [metaClassSelectors at: aClass soleInstance name ifAbsent: [^self]]
		ifFalse: [classSelectors at: aClass name ifAbsent: [^self]]) 
			remove: aSelector
			ifAbsent: []! !

!RBSelectorEnvironment methodsFor: 'accessing' stamp: ''!
selectorsForClass: aClass do: aBlock 
	^(self privateSelectorsForClass: aClass) 
		do: [:each | (aClass includesSelector: each) ifTrue: [aBlock value: each]]! !

!RBSelectorEnvironment methodsFor: 'printing' stamp: ''!
storeOn: aStream 
	| classBlock |
	aStream
		nextPutAll: '((';
		nextPutAll: self class name;
		nextPutAll: ' onEnvironment: '.
	environment storeOn: aStream.
	aStream
		nextPut: $);
		nextPutAll: ' classes: #('.
	classBlock := 
			[:key :value | 
			aStream
				nextPutAll: '#(';
				nextPutAll: key;
				nextPutAll: ' #('.
			value do: 
					[:each | 
					aStream
						nextPutAll: each;
						nextPut: $ ].
			aStream
				nextPutAll: '))';
				cr].
	classSelectors keysAndValuesDo: classBlock.
	aStream nextPutAll: ') metaClasses: #('.
	metaClassSelectors keysAndValuesDo: classBlock.
	aStream nextPutAll: '))'! !

RBBrowserEnvironmentWrapper subclass: #RBVariableEnvironment
	instanceVariableNames: 'instanceVariables instanceVariableReaders instanceVariableWriters classVariables selectorCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactoring-Environment'!

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 19:35'!
on: anEnvironment readersOfInstanceVariable: aString in: aClass 
	| newEnv |
	newEnv := (self onEnvironment: anEnvironment)
				label: 'Readers of ''' , aString , ''' in ' , aClass name;
				yourself.
	(aClass whichClassDefinesInstVar: aString) withAllSubclassesDo: 
			[:cls | 
			(cls whichSelectorsReallyRead: aString) isEmpty
				ifFalse: [newEnv addClass: cls instanceVariableReader: aString]].
	^newEnv! !

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'md 1/17/2006 14:17'!
on: anEnvironment referencesToClassVariable: aSymbol in: aClass 
	| newEnv definingClass assoc |
	newEnv := (self onEnvironment: anEnvironment)
				label: 'References to ''' , aSymbol , ''' in ' , aClass name;
				yourself.
	definingClass := aClass whichClassDefinesClassVar: aSymbol.
	assoc := definingClass bindingOf: aSymbol.
	definingClass withAllSubclassesDo: 
			[:cls | 
			(cls whichSelectorsReferTo: assoc) isEmpty 
				ifFalse: [newEnv addClass: cls classVariable: aSymbol]].
	^newEnv! !

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 19:35'!
on: anEnvironment referencesToInstanceVariable: aString in: aClass 
	| newEnv |
	newEnv := (self onEnvironment: anEnvironment)
				label: 'References to ''' , aString , ''' in ' , aClass name;
				yourself.
	(aClass whichClassDefinesInstVar: aString) withAllSubclassesDo: 
			[:cls | 
			((cls whichSelectorsReallyRead: aString) isEmpty not 
				or: [(cls whichSelectorsAssign: aString) isEmpty not]) 
					ifTrue: [newEnv addClass: cls instanceVariable: aString]].
	^newEnv! !

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'nk 3/4/2005 13:17'!
on: anEnvironment writersOfInstanceVariable: aString in: aClass 
	| newEnv |
	newEnv := (self onEnvironment: anEnvironment)
				label: 'Writers of ''' , aString , ''' in ' , aClass name;
				yourself.
	(aClass whichClassDefinesInstVar: aString) withAllSubclassesDo: 
			[:cls | 
			(cls whichSelectorsAssign: aString) isEmpty
				ifFalse: [newEnv addClass: cls instanceVariableWriter: aString]].
	^newEnv! !

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/8/2011 20:25'!
readersOfInstanceVariable: aString in: aClass 
	^ self 
		on: RBBrowserEnvironment new
		readersOfInstanceVariable: aString
		in: aClass! !

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/8/2011 20:25'!
referencesToClassVariable: aSymbol in: aClass 
	^ self 
		on: RBBrowserEnvironment new
		referencesToClassVariable: aSymbol
		in: aClass! !

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/8/2011 20:25'!
referencesToInstanceVariable: aString in: aClass
	^ self 
		on: RBBrowserEnvironment new
		referencesToInstanceVariable: aString
		in: aClass! !

!RBVariableEnvironment class methodsFor: 'instance creation' stamp: 'lr 9/8/2011 20:25'!
writersOfInstanceVariable: aString in: aClass 
	^ self 
		on: RBBrowserEnvironment new
		writersOfInstanceVariable: aString
		in: aClass! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
accessorMethods
	^#(#instanceVariables #instanceVariableReaders #instanceVariableWriters #classVariables)! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
addClass: aClass classVariable: aSymbol 
	(classVariables at: aClass name ifAbsentPut: [Set new]) add: aSymbol.
	self flushCachesFor: aClass.
	self addSearchString: aSymbol! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
addClass: aClass instanceVariable: aString 
	(instanceVariables at: aClass name ifAbsentPut: [Set new]) add: aString.
	self flushCachesFor: aClass.
	self addSearchString: aString! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
addClass: aClass instanceVariableReader: aString 
	(instanceVariableReaders at: aClass name ifAbsentPut: [Set new]) 
		add: aString.
	self flushCachesFor: aClass.
	self addSearchString: aString! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
addClass: aClass instanceVariableWriter: aString 
	(instanceVariableWriters at: aClass name ifAbsentPut: [Set new]) 
		add: aString.
	self flushCachesFor: aClass.
	self addSearchString: aString! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
allClassesDo: aBlock 
	| classes instVarBlock |
	classes := Set new.
	instVarBlock := 
			[:each | 
			| class |
			class := self classForName: each.
			classes addAll: class withAllSubclasses].
	instanceVariables keysDo: instVarBlock.
	instanceVariableReaders keysDo: instVarBlock.
	instanceVariableWriters keysDo: instVarBlock.
	classVariables keysDo: 
			[:each | 
			| class |
			class := self classForName: each.
			class notNil 
				ifTrue: 
					[classes
						addAll: class withAllSubclasses;
						addAll: class class withAllSubclasses]].
	classes do: aBlock! !

!RBVariableEnvironment methodsFor: 'private' stamp: 'lr 3/20/2011 11:18'!
classForName: aString 
	| name isMeta class |
	isMeta := aString includes: $ .
	name := (isMeta 
		ifTrue: [ aString copyFrom: 1 to: (aString size - 6 max: 1) ]
		ifFalse: [ aString ]) asSymbol.
	class := self systemDictionary at: name ifAbsent: [ nil ].
	^ (class notNil and: [ isMeta ])
		ifTrue: [ class class ]
		ifFalse: [ class ]! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
classNamesWithVariables
	| classNames |
	classNames := Set new.
	classNames
		addAll: instanceVariables keys;
		addAll: instanceVariableReaders keys;
		addAll: instanceVariableWriters keys;
		addAll: classVariables keys.
	^classNames! !

!RBVariableEnvironment methodsFor: 'private' stamp: 'lr 11/25/2009 08:32'!
classVariableSelectorsFor: aClass 
	| selectors classVars |
	selectors := Set new.
	classVars := Set new.
	classVariables keysDo: 
			[:each | 
			| cls |
			cls := self classForName: each.
			(cls notNil and: [aClass theNonMetaClass includesBehavior: cls]) 
				ifTrue: [classVars addAll: (classVariables at: each)]].
	classVars do: 
			[:each | 
			| binding |
			binding := aClass bindingOf: each.
			binding notNil 
				ifTrue: [selectors addAll: (aClass whichSelectorsReferTo: binding)]].
	^selectors! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
classVariables
	^classVariables! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
classVariables: anObject
	classVariables := anObject! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
classVariablesFor: aClass 
	^classVariables at: aClass name ifAbsent: [#()]! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
computeSelectorCacheFor: aClass 
	^(self instanceVariableSelectorsFor: aClass)
		addAll: (self classVariableSelectorsFor: aClass);
		yourself! !

!RBVariableEnvironment methodsFor: 'copying' stamp: ''!
copyDictionary: aDictionary 
	| copy |
	copy := Dictionary new: aDictionary size.
	aDictionary keysAndValuesDo: [:key :value | copy at: key put: value].
	^copy! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: 'lr 9/8/2011 20:32'!
environmentForClassVariable: aSymbol in: aClass 
	| selectorEnvironment assoc block |
	selectorEnvironment := RBSelectorEnvironment onEnvironment: self.
	selectorEnvironment addSearchString: aSymbol.
	((classVariables at: aClass name ifAbsent: [#()]) includes: aSymbol) 
		ifFalse: [^selectorEnvironment].
	assoc := aClass bindingOf: aSymbol.
	block := 
			[:each | 
			(each whichSelectorsReferTo: assoc) 
				do: [:sel | selectorEnvironment addClass: each selector: sel]].
	aClass withAllSubAndSuperclassesDo: 
			[:each | 
			block
				value: each;
				value: each class].
	^selectorEnvironment! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: 'lr 9/8/2011 20:32'!
environmentForInstanceVariable: aString in: aClass 
	| selectorEnvironment isReader isWriter |
	selectorEnvironment := RBSelectorEnvironment onEnvironment: self.
	selectorEnvironment addSearchString: aString.
	isReader := isWriter := false.
	((instanceVariables at: aClass name ifAbsent: [#()]) includes: aString) 
		ifTrue: 
			[isReader := true.
			isWriter := true].
	((instanceVariableWriters at: aClass name ifAbsent: [#()]) 
		includes: aString) ifTrue: [isWriter := true].
	((instanceVariableReaders at: aClass name ifAbsent: [#()]) 
		includes: aString) ifTrue: [isReader := true].
	aClass withAllSubAndSuperclassesDo:
			[:each | 
			isWriter 
				ifTrue: 
					[(each whichSelectorsAssign: aString) 
						do: [:sel | selectorEnvironment addClass: each selector: sel]].
			isReader 
				ifTrue: 
					[(each whichSelectorsReallyRead: aString) 
						do: [:sel | selectorEnvironment addClass: each selector: sel]]].
	^selectorEnvironment! !

!RBVariableEnvironment methodsFor: 'private' stamp: 'lr 11/25/2009 00:40'!
flushCachesFor: aClass 
	selectorCache isNil ifTrue: [ ^ self] .
	aClass theNonMetaClass withAllSubclasses do: [ :each | 
		selectorCache
			removeKey: each ifAbsent: [];
			removeKey: each class ifAbsent: [] ]! !

!RBVariableEnvironment methodsFor: 'testing' stamp: ''!
includesCategory: aCategory 
	^(self classNamesFor: aCategory) isEmpty not! !

!RBVariableEnvironment methodsFor: 'testing' stamp: 'lr 2/9/2008 10:51'!
includesClass: aClass 
	(super includesClass: aClass) ifFalse: [^false].
	(instanceVariables includesKey: aClass name) ifTrue: [^true].
	(classVariables includesKey: aClass name) ifTrue: [^true].
	^((self selectorCacheFor: aClass) 
		detect: [:each | self includesSelector: each in: aClass]
		ifNone: [nil]) notNil! !

!RBVariableEnvironment methodsFor: 'testing' stamp: ''!
includesProtocol: aProtocol in: aClass 
	^(self selectorsFor: aProtocol in: aClass) isEmpty not! !

!RBVariableEnvironment methodsFor: 'testing' stamp: ''!
includesSelector: aSymbol in: aClass 
	^(environment includesSelector: aSymbol in: aClass) 
		and: [(self selectorCacheFor: aClass) includes: aSymbol]! !

!RBVariableEnvironment methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	instanceVariables := Dictionary new.
	classVariables := Dictionary new.
	instanceVariableReaders := Dictionary new.
	instanceVariableWriters := Dictionary new! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
instanceVariableReaders
	^instanceVariableReaders! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
instanceVariableReaders: anObject
	instanceVariableReaders := anObject! !

!RBVariableEnvironment methodsFor: 'private' stamp: 'lr 4/29/2010 19:35'!
instanceVariableSelectorsFor: aClass 
	| selectors |
	selectors := Set new.
	#(#instanceVariables #instanceVariableReaders #instanceVariableWriters) 
		with: #(#whichSelectorsAccess: #whichSelectorsReallyRead: #whichSelectorsAssign:)
		do: [:var :sel | 
			| instVars |
			instVars := Set new.
			(self perform: var) keysDo: 
					[:each | 
					| cls |
					cls := self classForName: each.
					(cls notNil and: [aClass includesBehavior: cls]) 
						ifTrue: [instVars addAll: ((self perform: var) at: each)]].
			instVars do: [:each | selectors addAll: (aClass perform: sel with: each)]].
	^selectors! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
instanceVariableWriters
	^instanceVariableWriters! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
instanceVariableWriters: anObject
	instanceVariableWriters := anObject! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
instanceVariables
	^instanceVariables! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
instanceVariables: anObject
	instanceVariables := anObject! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
instanceVariablesFor: aClass 
	| vars name |
	vars := Set new.
	name := aClass name.
	vars
		addAll: (instanceVariables at: name ifAbsent: [#()]);
		addAll: (instanceVariableReaders at: name ifAbsent: [#()]);
		addAll: (instanceVariableWriters at: name ifAbsent: [#()]).
	^vars! !

!RBVariableEnvironment methodsFor: 'testing' stamp: 'TestRunner 1/3/2010 11:29'!
isEmpty
	^ self accessorMethods allSatisfy: [ :each | (self perform: each) isEmpty ]! !

!RBVariableEnvironment methodsFor: 'testing' stamp: 'lr 1/3/2010 11:11'!
isVariableEnvironment
	^ true! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
numberVariables
	^self accessorMethods inject: 0
		into: [:sum :each | sum + ((self perform: each) inject: 0 into: [:s :e | s + e size])]! !

!RBVariableEnvironment methodsFor: 'copying' stamp: ''!
postCopy
	super postCopy.
	instanceVariables := self copyDictionary: instanceVariables.
	instanceVariableReaders := self copyDictionary: instanceVariableReaders.
	instanceVariableWriters := self copyDictionary: instanceVariableWriters.
	classVariables := self copyDictionary: classVariables.
	selectorCache := nil! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
problemCount
	^self numberVariables! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
removeClass: aClass classVariable: aSymbol 
	| vars |
	vars := classVariables at: aClass name ifAbsent: [Set new].
	vars remove: aSymbol ifAbsent: [].
	vars isEmpty ifTrue: [classVariables removeKey: aClass name ifAbsent: []].
	self flushCachesFor: aClass! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
removeClass: aClass instanceVariable: aString 
	| vars |
	vars := instanceVariables at: aClass name ifAbsent: [Set new].
	vars remove: aString ifAbsent: [].
	vars isEmpty 
		ifTrue: [instanceVariables removeKey: aClass name ifAbsent: []].
	self flushCachesFor: aClass! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
removeClass: aClass instanceVariableReader: aString 
	| vars |
	vars := instanceVariableReaders at: aClass name ifAbsent: [Set new].
	vars remove: aString ifAbsent: [].
	vars isEmpty 
		ifTrue: [instanceVariableReaders removeKey: aClass name ifAbsent: []].
	self flushCachesFor: aClass! !

!RBVariableEnvironment methodsFor: 'accessing' stamp: ''!
removeClass: aClass instanceVariableWriter: aString 
	| vars |
	vars := instanceVariableWriters at: aClass name ifAbsent: [Set new].
	vars remove: aString ifAbsent: [].
	vars isEmpty 
		ifTrue: [instanceVariableWriters removeKey: aClass name ifAbsent: []].
	self flushCachesFor: aClass! !

!RBVariableEnvironment methodsFor: 'private' stamp: ''!
selectorCache
	^selectorCache isNil 
		ifTrue: [selectorCache := Dictionary new]
		ifFalse: [selectorCache]! !

!RBVariableEnvironment methodsFor: 'private' stamp: 'lr 11/25/2009 00:42'!
selectorCacheFor: aClass 
	^self selectorCache at: aClass name ifAbsentPut: [ self computeSelectorCacheFor: aClass ]! !

!RBVariableEnvironment methodsFor: 'printing' stamp: 'lr 4/29/2010 19:06'!
storeOn: aStream 
	aStream
		nextPut: $(;
		nextPutAll: self class name;
		nextPutAll: ' new '.
	self accessorMethods do: [ :each | 
		aStream
			nextPutAll: each;
			nextPutAll: ': '.
		(self perform: each) storeOn: aStream.
		aStream nextPutAll: '; '].
	aStream nextPutAll: 'yourself)'! !
